"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addReExport = exports.addExport = exports.resolveSourceFile = exports.registerOtherExportOrReExport = exports.registerDirectExport = exports.registerNamespaceReExport = exports.registerDirectReExport = void 0;
const path_1 = __importDefault(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const utils_1 = require("../utils");
function registerDirectReExport(project, currentModule, decl) {
    const currentSourceFile = decl.getSourceFile().fileName;
    const sourceFile = resolveSourceFile(project.extractor, currentSourceFile, decl.text);
    if (!sourceFile)
        return;
    const moduleOfSource = project.getOrCreateModule(sourceFile.fileName);
    if (moduleOfSource !== currentModule)
        project.visitor(sourceFile, moduleOfSource);
    else
        project.visitor(sourceFile, currentModule);
    addReExport(currentModule, currentSourceFile, sourceFile.fileName, {
        module: moduleOfSource.ref,
        references: [],
        sameModule: currentModule === moduleOfSource
    });
}
exports.registerDirectReExport = registerDirectReExport;
function registerNamespaceReExport(project, currentModule, val) {
    const namespaceName = val.declarations[0].name.text;
    const aliased = project.resolveAliasedSymbol(val);
    if (!aliased.declarations || !aliased.declarations.length)
        return;
    const fileNameOfSource = project.resolveSymbolFileName(aliased);
    const mod = project.getOrCreateModule(fileNameOfSource);
    project.visitor(aliased, mod);
    addReExport(currentModule, val.declarations[0].getSourceFile().fileName, fileNameOfSource, {
        namespace: namespaceName,
        module: mod.ref,
        references: [],
        sameModule: currentModule === mod
    });
}
exports.registerNamespaceReExport = registerNamespaceReExport;
function registerDirectExport(fileName, currentModule, ref) {
    if (ref.kind === 12)
        return;
    addExport(currentModule, fileName, ref);
}
exports.registerDirectExport = registerDirectExport;
function registerOtherExportOrReExport(project, currentModule, val) {
    const firstDecl = val.declarations[0];
    const thisSourceFile = firstDecl.getSourceFile().fileName;
    const realObj = project.resolveAliasedSymbol(val);
    let originSourceFile;
    if (firstDecl.parent.parent.moduleSpecifier) {
        originSourceFile = resolveSourceFile(project.extractor, thisSourceFile, firstDecl.parent.parent.moduleSpecifier.text);
        if (!originSourceFile)
            return;
    }
    else
        originSourceFile = realObj.declarations[0].getSourceFile();
    const originModule = project.getOrCreateModule(originSourceFile.fileName);
    project.visitor(originSourceFile, originModule);
    if (typescript_1.default.isSourceFile(realObj.declarations[0])) {
        if (originModule.exports.index?.reExports.some(ex => ex.namespace === val.name))
            addReExport(currentModule, thisSourceFile, originSourceFile.fileName, {
                module: originModule.ref,
                references: [],
                sameModule: currentModule === originModule,
                reExportsOfReExport: val.name
            });
        else
            addReExport(currentModule, thisSourceFile, originSourceFile.fileName, {
                namespace: val.name,
                module: originModule.ref,
                references: [],
                sameModule: currentModule === originModule
            });
        return;
    }
    const reference = project.extractor.refs.get(realObj);
    const alias = (val.name === realObj.name) ? undefined : val.name;
    const thisFileName = (0, utils_1.getFilenameFromPath)(thisSourceFile);
    const originFileName = (0, utils_1.getFilenameFromPath)(originSourceFile.fileName);
    if (!reference) {
        currentModule.exports[thisFileName]?.reExports.push({
            module: originModule.ref,
            filename: originFileName === "index" ? undefined : originFileName,
            references: [],
            sameModule: currentModule === originModule,
            reExportsOfReExport: val.name
        });
        return;
    }
    if (reference.kind === 12)
        return;
    if (!currentModule.exports[thisFileName])
        currentModule.exports[thisFileName] = {
            exports: [], reExports: [
                {
                    module: originModule.ref,
                    filename: originFileName === "index" ? undefined : originFileName,
                    references: [{ ...reference, alias }],
                    sameModule: currentModule === originModule
                }
            ]
        };
    else {
        const reExport = currentModule.exports[thisFileName].reExports.find(r => r.module === originModule.ref && r.filename === originFileName);
        if (!reExport)
            currentModule.exports[thisFileName].reExports.push({
                module: originModule.ref,
                filename: originFileName === "index" ? undefined : originFileName,
                references: [{ ...reference, alias }],
                sameModule: currentModule === originModule
            });
        else
            reExport?.references.push({ ...reference, alias });
    }
}
exports.registerOtherExportOrReExport = registerOtherExportOrReExport;
function resolveSourceFile(extractor, filePath, relative) {
    let res;
    if (path_1.default.isAbsolute(filePath)) {
        const p = path_1.default.join(filePath, "../");
        res = extractor.program.getSourceFile(path_1.default.join(p, `${relative}.ts`));
        if (!res)
            res = extractor.program.getSourceFile(path_1.default.join(p, `${relative}/index.ts`));
        if (!res)
            res = extractor.program.getSourceFile(path_1.default.join(p, `${relative.slice(0, -3)}.ts`));
        if (!res)
            res = extractor.program.getSourceFile(path_1.default.join(p, `${relative}.tsx`));
        if (!res)
            res = extractor.program.getSourceFile(path_1.default.join(p, `${relative}/index.tsx`));
        if (!res)
            res = extractor.program.getSourceFile(path_1.default.join(p, `${relative.slice(0, -3)}.tsx`));
        if (!res)
            res = extractor.program.getSourceFile(path_1.default.join(p, `${relative}.d.ts`));
        if (!res)
            res = extractor.program.getSourceFile(path_1.default.join(p, `${relative}/index.d.ts`));
        if (!res)
            res = extractor.program.getSourceFile(path_1.default.join(p, `${relative.slice(0, -3)}.d.ts`));
    }
    else {
        const p = path_1.default.join(process.cwd(), filePath, "../");
        res = extractor.program.getSourceFile(path_1.default.join(p, `${relative}.ts`));
        if (!res)
            res = extractor.program.getSourceFile(path_1.default.join(p, `${relative}/index.ts`));
        if (!res)
            res = extractor.program.getSourceFile(path_1.default.join(p, `${relative.slice(0, -3)}.ts`));
        if (!res)
            res = extractor.program.getSourceFile(path_1.default.join(p, `${relative}.tsx`));
        if (!res)
            res = extractor.program.getSourceFile(path_1.default.join(p, `${relative}/index.tsx`));
        if (!res)
            res = extractor.program.getSourceFile(path_1.default.join(p, `${relative.slice(0, -3)}.tsx`));
        if (!res)
            res = extractor.program.getSourceFile(path_1.default.join(p, `${relative}.d.ts`));
        if (!res)
            res = extractor.program.getSourceFile(path_1.default.join(p, `${relative}/index.d.ts`));
        if (!res)
            res = extractor.program.getSourceFile(path_1.default.join(p, `${relative.slice(0, -3)}.d.ts`));
    }
    return res;
}
exports.resolveSourceFile = resolveSourceFile;
function addExport(module, fileName, ex) {
    const last = (0, utils_1.getFilenameFromPath)(fileName);
    if (!module.exports[last])
        module.exports[last] = { exports: [ex], reExports: [] };
    else
        module.exports[last].exports.push(ex);
}
exports.addExport = addExport;
function addReExport(module, fileName, origin, ex) {
    const last = (0, utils_1.getFilenameFromPath)(fileName);
    const originLast = (0, utils_1.getFilenameFromPath)(origin);
    if (originLast !== "index")
        ex.filename = originLast;
    if (!module.exports[last])
        module.exports[last] = { exports: [], reExports: [ex] };
    else
        module.exports[last].reExports.push(ex);
}
exports.addReExport = addReExport;
