"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createHost = void 0;
const path_1 = __importDefault(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const utils_1 = require("../utils");
function createHost(options, customModules, extractorOptions, cwd) {
    const defaultHost = extractorOptions.compilerHost ? extractorOptions.compilerHost(options) : typescript_1.default.createCompilerHost(options, true);
    defaultHost.resolveModuleNames = (mods, file) => {
        const res = [];
        for (const module of mods) {
            let part = module;
            let rest = "";
            if (module.startsWith("@")) {
                const secondSlash = module.indexOf("/", module.indexOf("/") + 1);
                if (secondSlash !== -1) {
                    part = module.slice(0, secondSlash);
                    if (extractorOptions.ignoreFolderNames)
                        rest = (0, utils_1.removePartOfPath)(module.slice(secondSlash).split("/"), extractorOptions.ignoreFolderNames) + ".ts";
                    else
                        rest = module.slice(secondSlash) + ".ts";
                }
            }
            else if (!part.startsWith(".") && part.includes("/")) {
                const firstSlash = part.indexOf("/");
                part = part.slice(0, firstSlash);
                if (extractorOptions.ignoreFolderNames)
                    rest = (0, utils_1.removePartOfPath)(module.slice(firstSlash).split("/"), extractorOptions.ignoreFolderNames) + ".ts";
                else
                    rest = module.slice(firstSlash) + ".ts";
            }
            if (customModules.has(part)) {
                let resolvedFileName;
                const customModulePart = customModules.get(part);
                if (rest)
                    resolvedFileName = (path_1.default.isAbsolute(customModulePart) ? path_1.default.join(path_1.default.parse(customModulePart).dir, rest) : path_1.default.join(cwd, path_1.default.parse(customModulePart).dir, rest)).replace(/\\/g, "/");
                else
                    resolvedFileName = (path_1.default.isAbsolute(customModulePart) ? customModulePart : path_1.default.join(cwd, customModulePart)).replace(/\\/g, "/");
                res.push({
                    resolvedFileName,
                    isExternalLibraryImport: false,
                    extension: typescript_1.default.Extension.Ts
                });
            }
            else {
                const result = typescript_1.default.resolveModuleName(module, file, options, { fileExists: defaultHost.fileExists, readFile: defaultHost.readFile });
                if (result.resolvedModule && !result.resolvedModule.isExternalLibraryImport && (result.resolvedModule.extension === typescript_1.default.Extension.Ts
                    || result.resolvedModule.extension === typescript_1.default.Extension.Tsx
                    || result.resolvedModule.extension === typescript_1.default.Extension.Dts))
                    res.push(result.resolvedModule);
                else
                    res.push(undefined);
            }
        }
        return res;
    };
    return defaultHost;
}
exports.createHost = createHost;
