import ts from "typescript";
import { ObjectProperty, TypescriptExtractor } from ".";
import { PackageJSON } from "../utils";
import { FunctionParameter, JSDocData, Loc, Module, Reference, ReferenceType, Type, TypeParameter } from "./structure";
export declare class Project {
    repository?: string;
    readme?: string;
    homepage?: string;
    version?: string;
    module: Module;
    extractor: TypescriptExtractor;
    baseDir: string;
    root: string;
    tsconfig?: ts.CompilerOptions;
    private ignoreNamespaceMembers?;
    private idAcc;
    constructor({ folderPath, extractor, packageJSON, tsconfig }: {
        folderPath: Array<string>;
        extractor: TypescriptExtractor;
        packageJSON: PackageJSON;
        tsconfig?: ts.CompilerOptions;
    });
    visitor(sourceFile: ts.SourceFile | ts.Symbol, currentModule: Module): void;
    getOrCreateModule(source: string): Module;
    forEachModule<R>(cb: (module: Module) => R | undefined, module?: Module): R | undefined;
    handleSymbol(val: ts.Symbol, currentModule?: Module, isCached?: boolean): ReferenceType | undefined;
    handleClassDecl(symbol: ts.Symbol, currentModule: Module, isCached?: boolean): ReferenceType | undefined;
    handleInterfaceDecl(sym: ts.Symbol, currentModule: Module, isCached?: boolean): ReferenceType | undefined;
    handleEnumDecl(sym: ts.Symbol, currentModule: Module, isCached?: boolean): ReferenceType | undefined;
    handleTypeAliasDecl(sym: ts.Symbol, currentModule: Module, isCached?: boolean): ReferenceType | undefined;
    handleNamespaceDecl(symbol: ts.Symbol, currentModule: Module, isCached?: boolean): ReferenceType | undefined;
    handleVariableDecl(sym: ts.Symbol, currentModule: Module, isCached?: boolean): ReferenceType | undefined;
    handleFunctionDecl(sym: ts.Symbol, currentModule: Module, isCached?: boolean): ReferenceType | undefined;
    resolveSymbolOrStr(node: ts.Node, typeArguments?: Array<Type>): Type;
    resolveSymbol(sym: ts.Symbol, typeArguments?: Array<Type>): Reference;
    resolveType(type: ts.Node): Type;
    resolveReturnType(fn: ts.MethodDeclaration | ts.FunctionDeclaration | ts.GetAccessorDeclaration | ts.SetAccessorDeclaration | ts.FunctionExpression | ts.ArrowFunction): Type | undefined;
    resolveTypeParameters(generic: ts.TypeParameterDeclaration): TypeParameter;
    resolveObjectProperty(prop: ts.TypeElement): ObjectProperty;
    resolveHeritage(param: ts.ExpressionWithTypeArguments): Type;
    resolveParameter(param: ts.ParameterDeclaration): FunctionParameter;
    resolveExpressionToType(exp: ts.Node, tryType?: boolean): Type;
    resolveTypeType(type: ts.Type): Type;
    resolveAliasedSymbol(symbol: ts.Symbol): ts.Symbol;
    getJSDocCommentOfParam(node: ts.ParameterDeclaration): string | undefined;
    resolveSymbolFileName(sym: ts.Symbol): string;
    getJSDocData(node: ts.Node): Array<JSDocData> | undefined;
    getJSDocDataRaw(node: ts.Node): Array<JSDocData> | undefined;
    jsDocToJsDocData(jsDoc: Array<ts.JSDoc>): Array<JSDocData>;
    getLOC(currentModule: Module, node: ts.Node, includeFilename?: boolean): Loc;
    moduleToJSON(module?: Module): Record<string, unknown>;
    toJSON(): Record<string, unknown>;
    isInternalNode(node: ts.Node): boolean | undefined;
}
//# sourceMappingURL=Project.d.ts.map