"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReferenceManager = void 0;
const typescript_1 = __importDefault(require("typescript"));
class ReferenceManager extends Map {
    namedExternals;
    unnamedExternals;
    constructor(externals) {
        super();
        this.namedExternals = new Map();
        this.unnamedExternals = [];
        if (externals) {
            for (const external of externals) {
                if (external.baseName)
                    this.namedExternals.set(external.baseName, external);
                else
                    this.unnamedExternals.push(external);
            }
        }
    }
    findUnnamedExternal(symbol, source, other) {
        for (const external of this.unnamedExternals) {
            const res = external.run(symbol, source, other);
            if (res)
                return { name: symbol, kind: 11, ...res };
        }
        return;
    }
    findExternal(symbol, source, realName) {
        if (this.has(symbol))
            return this.get(symbol);
        let name = realName || symbol.name;
        if (!source && symbol.declarations && symbol.declarations.length && !symbol.declarations[0].getSourceFile().isDeclarationFile) {
            const decl = symbol.declarations[0];
            if (typescript_1.default.isImportClause(decl))
                source = decl.parent.moduleSpecifier.text;
            else if (typescript_1.default.isImportSpecifier(decl)) {
                source = decl.parent.parent.parent.moduleSpecifier.text;
                if (decl.propertyName)
                    name = decl.propertyName.text;
            }
            else if (typescript_1.default.isNamespaceImport(decl))
                source = decl.parent.parent.moduleSpecifier.text;
        }
        if (source) {
            const path = source.split("/");
            const first = path.shift();
            if (first && this.namedExternals.has(first)) {
                const res = { name, kind: 11, ...this.namedExternals.get(first).run(name, source) };
                if (!realName)
                    this.set(symbol, res);
                return res;
            }
        }
        const unnamed = this.findUnnamedExternal(name, source, symbol.name);
        if (unnamed && !realName)
            this.set(symbol, unnamed);
        return unnamed;
    }
    findByName(name) {
        for (const [, ref] of this) {
            if (ref.name === name)
                return ref;
        }
        return;
    }
}
exports.ReferenceManager = ReferenceManager;
