import ts from "typescript";
import { Project } from "./Project";
import { ExternalReference, ReferenceManager } from "./ReferenceManager";
import { Module } from "./structure";
export declare abstract class FileObjectCache {
    abstract has(absolute: string): boolean;
}
export interface TypescriptExtractorSettings {
    entryPoints: Array<string>;
    ignoreFolderNames?: Array<string>;
    maxConstantTextLength?: number;
    externals?: Array<ExternalReference>;
    passthroughModules?: Array<string>;
    cwd?: string;
    fileCache?: FileObjectCache;
    refs?: ReferenceManager;
    tsconfig?: string;
    stripInternal?: boolean;
    branchName?: string;
    compilerHost?: (tsconfig: ts.CompilerOptions) => ts.CompilerHost;
    documentImports?: boolean;
}
export declare class TypescriptExtractor {
    settings: TypescriptExtractorSettings;
    checker: ts.TypeChecker;
    program: ts.Program;
    refs: ReferenceManager;
    moduleCache: Record<string, Module>;
    fileCache: Map<string, boolean | undefined>;
    splitCwd: Array<string>;
    constructor(settings: TypescriptExtractorSettings);
    isCachedFile(fileName: string): boolean | undefined;
    run(): Array<Project>;
}
export * from "./Host";
export * from "./Project";
export * from "./ReferenceManager";
export * from "./structure";
export * as ExtractorUtils from "./utils";
//# sourceMappingURL=index.d.ts.map