"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExtractorUtils = exports.TypescriptExtractor = exports.FileObjectCache = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const utils_1 = require("../utils");
const Host_1 = require("./Host");
const Project_1 = require("./Project");
const ReferenceManager_1 = require("./ReferenceManager");
class FileObjectCache {
}
exports.FileObjectCache = FileObjectCache;
class TypescriptExtractor {
    settings;
    checker;
    program;
    refs;
    moduleCache;
    fileCache;
    splitCwd;
    constructor(settings) {
        this.settings = settings;
        this.refs = settings.refs || new ReferenceManager_1.ReferenceManager(settings.externals);
        this.moduleCache = {};
        this.fileCache = new Map();
    }
    isCachedFile(fileName) {
        if (this.fileCache.has(fileName))
            return this.fileCache.get(fileName);
        const res = this.settings.fileCache?.has(fileName);
        this.fileCache.set(fileName, res);
        return res;
    }
    run() {
        const cwd = this.settings.cwd || process.cwd();
        this.splitCwd = cwd.split(path_1.default.sep);
        let tsconfig = {};
        if (this.settings.tsconfig) {
            if (this.settings.tsconfig !== "none") {
                const info = typescript_1.default.parseConfigFileTextToJson("tsconfig.json", fs_1.default.readFileSync(path_1.default.join(cwd, this.settings.tsconfig), "utf-8"));
                if (info.error)
                    throw new Error(typescript_1.default.flattenDiagnosticMessageText(info.error.messageText, "\n"));
                tsconfig = typescript_1.default.convertCompilerOptionsFromJson(info.config.compilerOptions, cwd).options;
            }
        }
        else {
            const tsconfigPath = typescript_1.default.findConfigFile(cwd, (file) => fs_1.default.existsSync(file), "tsconfig.json");
            if (tsconfigPath) {
                const configRes = typescript_1.default.parseConfigFileTextToJson("tsconfig.json", fs_1.default.readFileSync(tsconfigPath, "utf-8"));
                if (configRes.error)
                    throw new Error(typescript_1.default.flattenDiagnosticMessageText(configRes.error.messageText, "\n"));
                if (configRes.config)
                    tsconfig = typescript_1.default.convertCompilerOptionsFromJson(configRes.config.compilerOptions, cwd).options;
            }
        }
        if (!tsconfig || Object.keys(tsconfig).length === 0) {
            tsconfig = {
                skipLibCheck: true,
                module: typescript_1.default.ModuleKind.CommonJS,
                target: typescript_1.default.ScriptTarget.ES2016
            };
        }
        if (typeof this.settings.stripInternal === "undefined")
            this.settings.stripInternal = tsconfig.stripInternal;
        tsconfig.types = [];
        const packagesMap = new Map();
        const packageJSONs = new Map();
        const host = (0, Host_1.createHost)(tsconfig, packagesMap, this.settings, cwd);
        for (let i = 0; i < this.settings.entryPoints.length; i++) {
            let entryPoint = this.settings.entryPoints[i];
            if (!entryPoint.endsWith("ts") && !entryPoint.endsWith("tsx"))
                entryPoint += ".ts";
            if (!path_1.default.isAbsolute(entryPoint))
                entryPoint = path_1.default.join(cwd, entryPoint);
            if (!host.fileExists(entryPoint))
                throw new Error(`Couldn't find file '${entryPoint}'`);
            const packageJSON = (0, utils_1.findPackageJSON)(entryPoint);
            if (!packageJSON)
                throw new Error("Couldn't find package.json file for one of the entry points");
            if (!packageJSON.contents.name)
                throw new Error("One of the entry points' package.json is missing a name");
            packagesMap.set(packageJSON.contents.name, entryPoint);
            const tsconfig = typescript_1.default.findConfigFile(packageJSON.path, (file) => fs_1.default.existsSync(file), "tsconfig.json");
            packageJSONs.set(entryPoint, [packageJSON, tsconfig ? typescript_1.default.parseConfigFileTextToJson("package.json", fs_1.default.readFileSync(tsconfig, "utf-8")).config.compilerOptions : undefined]);
            this.settings.entryPoints[i] = entryPoint;
        }
        this.program = typescript_1.default.createProgram(this.settings.entryPoints, tsconfig, host);
        this.checker = this.program.getTypeChecker();
        const projects = [];
        for (const entryPoint of this.settings.entryPoints) {
            const sourceFile = this.program.getSourceFile(entryPoint);
            if (!sourceFile)
                continue;
            const [packageJSON, tsconfig] = packageJSONs.get(entryPoint);
            const project = new Project_1.Project({ folderPath: (0, utils_1.removePartOfEndOfPath)(sourceFile.fileName, this.splitCwd), extractor: this, packageJSON, tsconfig });
            projects.push(project);
            project.visitor(sourceFile, project.module);
        }
        return projects;
    }
}
exports.TypescriptExtractor = TypescriptExtractor;
__exportStar(require("./Host"), exports);
__exportStar(require("./Project"), exports);
__exportStar(require("./ReferenceManager"), exports);
__exportStar(require("./structure"), exports);
exports.ExtractorUtils = __importStar(require("./utils"));
