"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findByPath = exports.findByNameWithModule = exports.findOfKindInModule = exports.findDeclOfType = exports.findDeclWithModule = exports.forEachModule = void 0;
function forEachModule(module, cb) {
    const firstCb = cb(module);
    if (firstCb)
        return firstCb;
    for (const [, mod] of module.modules) {
        const res = forEachModule(mod, cb);
        if (res)
            return res;
    }
    return undefined;
}
exports.forEachModule = forEachModule;
function findDeclWithModule(name, module) {
    return forEachModule(module, (module) => {
        const maxLoops = Math.max(module.classes.length, module.interfaces.length, module.enums.length, module.types.length, module.functions.length, module.constants.length);
        for (let i = 0 | 0; i < maxLoops; i++) {
            if (module.classes.length > i && module.classes[i].name === name)
                return module.classes[i];
            else if (module.interfaces.length > i && module.interfaces[i].name === name)
                return module.interfaces[i];
            else if (module.enums.length > i && module.enums[i].name === name)
                return module.enums[i];
            else if (module.types.length > i && module.types[i].name === name)
                return module.types[i];
            else if (module.functions.length > i && module.functions[i].name === name)
                return module.types[i];
            else if (module.constants.length > i && module.constants[i].name === name)
                return module.constants[i];
        }
        return;
    });
}
exports.findDeclWithModule = findDeclWithModule;
function findDeclOfType(name, module, kind) {
    switch (kind) {
        case "class":
            const classDecl = module.classes.find(c => c.name === name);
            if (classDecl)
                return classDecl;
            break;
        case "interface":
            const interfaceDecl = module.interfaces.find(intf => intf.name === name);
            if (interfaceDecl)
                return interfaceDecl;
            break;
        case "enum":
            const enumDecl = module.enums.find(e => e.name === name);
            if (enumDecl)
                return enumDecl;
            break;
        case "function":
            const fnDecl = module.functions.find(f => f.name === name);
            if (fnDecl)
                return fnDecl;
            break;
        case "type":
            const typeDecl = module.types.find(t => t.name === name);
            if (typeDecl)
                return typeDecl;
            break;
        case "constant":
            const constDecl = module.constants.find(c => c.name === name);
            if (constDecl)
                return constDecl;
            break;
        default:
            return;
    }
    return;
}
exports.findDeclOfType = findDeclOfType;
function findOfKindInModule(name, module, kind) {
    switch (kind) {
        case "class":
            if (module.classes.some(c => c.name === name))
                return { kind: 0, name, path: module.path };
            break;
        case "interface":
            if (module.interfaces.some(i => i.name === name))
                return { kind: 1, name, path: module.path };
            break;
        case "enum":
            if (module.enums.some(e => e.name === name))
                return { kind: 2, name, path: module.path };
            break;
        case "function":
            if (module.functions.some(f => f.name === name))
                return { kind: 3, name, path: module.path };
            break;
        case "type":
            if (module.types.some(t => t.name === name))
                return { kind: 5, name, path: module.path };
            break;
        case "constant":
            if (module.constants.some(c => c.name === name))
                return { kind: 4, name, path: module.path };
            break;
        case "module":
            if (module.modules.has(name))
                return { kind: 10, name, path: module.path };
            break;
        default:
            return;
    }
    return;
}
exports.findOfKindInModule = findOfKindInModule;
function findByNameWithModule(name, module) {
    return forEachModule(module, (module) => {
        const path = module.path;
        if (module.name === name)
            return { kind: 10, name, path };
        if (module.modules.has(name))
            return { kind: 10, name, path };
        const maxLoops = Math.max(module.classes.length, module.interfaces.length, module.enums.length, module.types.length, module.functions.length, module.constants.length);
        for (let i = 0 | 0; i < maxLoops; i++) {
            if (module.classes.length > i && module.classes[i].name === name)
                return { kind: 0, name, path };
            else if (module.interfaces.length > i && module.interfaces[i].name === name)
                return { kind: 1, name, path };
            else if (module.enums.length > i && module.enums[i].name === name)
                return { kind: 2, name, path };
            else if (module.types.length > i && module.types[i].name === name)
                return { kind: 5, name, path };
            else if (module.functions.length > i && module.functions[i].name === name)
                return { kind: 3, name, path };
            else if (module.constants.length > i && module.constants[i].name === name)
                return { kind: 4, name, path };
        }
        return;
    });
}
exports.findByNameWithModule = findByNameWithModule;
function findByPath(name, path, project) {
    let mod = project.module;
    for (const pathPart of path) {
        mod = mod.modules.get(pathPart);
        if (!mod)
            return;
    }
    if (mod.modules.has(name))
        return { kind: 10, name, path };
    const maxLoops = Math.max(mod.classes.length, mod.interfaces.length, mod.enums.length, mod.types.length, mod.functions.length, mod.constants.length);
    for (let i = 0 | 0; i < maxLoops; i++) {
        if (mod.classes.length > i && mod.classes[i].name === name)
            return { kind: 0, name, path };
        else if (mod.interfaces.length > i && mod.interfaces[i].name === name)
            return { kind: 1, name, path };
        else if (mod.enums.length > i && mod.enums[i].name === name)
            return { kind: 2, name, path };
        else if (mod.types.length > i && mod.types[i].name === name)
            return { kind: 5, name, path };
        else if (mod.functions.length > i && mod.functions[i].name === name)
            return { kind: 3, name, path };
        else if (mod.constants.length > i && mod.constants[i].name === name)
            return { kind: 4, name, path };
    }
    return;
}
exports.findByPath = findByPath;
