"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractMetadata = exports.hasBit = exports.getReadme = exports.getBranchName = exports.getRepository = exports.findPackageJSON = exports.getFilenameFromPath = exports.getLastItemFromPath = exports.removePartOfPath = exports.removePartOfEndOfPath = void 0;
const child_process_1 = require("child_process");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
function removePartOfEndOfPath(tsPath, toRemove) {
    const newPath = tsPath.split("/");
    if (toRemove.length > newPath.length)
        return newPath;
    return newPath.slice(toRemove.length);
}
exports.removePartOfEndOfPath = removePartOfEndOfPath;
function removePartOfPath(path, parts) {
    return path.filter(p => !parts.includes(p)).join("/");
}
exports.removePartOfPath = removePartOfPath;
function getLastItemFromPath(p) {
    return path_1.default.parse(p).base;
}
exports.getLastItemFromPath = getLastItemFromPath;
function getFilenameFromPath(p) {
    const name = path_1.default.parse(p);
    return name.base.slice(0, name.base.indexOf("."));
}
exports.getFilenameFromPath = getFilenameFromPath;
function findPackageJSON(basePath) {
    const pathToJson = path_1.default.join(basePath, "package.json");
    if (fs_1.default.existsSync(pathToJson))
        return { contents: JSON.parse(fs_1.default.readFileSync(pathToJson, "utf-8")), path: basePath };
    const newPath = path_1.default.join(basePath, "../");
    if (newPath === basePath)
        return undefined;
    return findPackageJSON(newPath);
}
exports.findPackageJSON = findPackageJSON;
function getRepository(packageJSON, branchName) {
    const repository = packageJSON.contents.repository;
    if (!repository)
        return;
    const branch = branchName || getBranchName(packageJSON.path);
    if (typeof repository === "string") {
        const [type, link] = repository.split(":");
        return `https://${type}.com/${link}/tree/${branch}`;
    }
    else {
        let { type, url } = repository;
        url = url.replace(new RegExp(`${type}:\/\/|ssh://${type}@`, "g"), "https://");
        return `${url.replace(new RegExp(`${type}\\+|\\.${type}`, "g"), "")}/tree/${branch}${repository.directory || ""}`;
    }
}
exports.getRepository = getRepository;
function getBranchName(path) {
    return (0, child_process_1.execSync)(`cd ${path} && git rev-parse --abbrev-ref HEAD`).slice(0, -1).toString("utf-8");
}
exports.getBranchName = getBranchName;
function getReadme(dir) {
    const pathToReadme = path_1.default.join(dir, "README.md");
    if (fs_1.default.existsSync(pathToReadme))
        return fs_1.default.readFileSync(pathToReadme, "utf-8");
    return;
}
exports.getReadme = getReadme;
function hasBit(num, bit) {
    return (num & bit) !== 0;
}
exports.hasBit = hasBit;
function extractMetadata(directory) {
    const packageJSON = findPackageJSON(directory);
    return {
        readme: getReadme(directory),
        homepage: packageJSON && packageJSON.contents.homepage,
        version: packageJSON && packageJSON.contents.version,
        repository: packageJSON && getRepository(packageJSON)
    };
}
exports.extractMetadata = extractMetadata;
